import { Component } from 'react';
import PropTypes from 'prop-types';

/**
 * Component that detects if you click outside of it
 */
export default class WithClickOutside extends Component {
    constructor(props) {
        super(props);

        this.state = {
            isClicked: props.isClicked,
        };
        this.setWrapperRef = this.setWrapperRef.bind(this);
        this.handleClickOutside = this.handleClickOutside.bind(this);
    }

    componentDidMount() {
        document.addEventListener('mousedown', this.handleClickOutside);
    }

    componentWillUnmount() {
        document.removeEventListener('mousedown', this.handleClickOutside);
    }

    /**
   * Set the wrapper ref
   */
    setWrapperRef(node) {
        this.wrapperRef = node;
    }

    /**
   If clicked on outside of element
   */
    handleClickOutside(event) {
        const { onClickOutside } = this.props;
        if (this.wrapperRef && !this.wrapperRef.contains(event.target)) {
            if (typeof (onClickOutside) === 'function')
                onClickOutside();

            this.setState({
                isClicked: true,
            });
        } else {
            this.setState({
                isClicked: false,
            });
        }
    }

    render() {
        return this.props.children(this.setWrapperRef, this.state.isClicked);
    }
}

WithClickOutside.defaultProps = {
    isClicked: false,
};

WithClickOutside.propTypes = {
    isClicked: PropTypes.bool,
    children: PropTypes.func.isRequired,
};
